/*
 * Main AA-RR Class
 *
 * $Id: AARRMain.java,v 1.1.1.1 2005/05/22 17:00:49 macias Exp $
 */
package es.rediris.aarr;

import es.rediris.aarr.protocol.ProtocolAdaptor;
import es.rediris.aarr.rules.*;


/** This is the main class of AARR. The following tasks are performed when this class is instantiated:
 * <ul>
 *   <li>A new {@link Diagnostic diagnostic } module is instantiated
 *   <li>A {@link ConfigurationProcessor configuration } module is instantiated and configuration is loaded
 *   <li>A {@link ProtocolAdaptor protocol adaptor } is instantiated
 *   <li>A {@link RuleProcessor rule processor } is instantiated
 *   <li>A {@link ProfileManager profile manager is } instantiated
 * </ul>
 * Find below more information on the methods contained within this class.
 *
 * @author kan
 * @author macias
 */
public class AARRMain {
    /** A string used to store the config filename if one is supplied */
    protected static String confFile = null;

    /** A <b>protocolBinding</b> built from a  {@link ProtocolAdaptor ProtocolAdaptor} */
    protected static ProtocolAdaptor protocolAdaptor;
    protected static RuleProcessor ruleProcessor;

    /** A {@link ConfigurationProcessor ConfigurationProcessor } object used to load and store a configuration */
    ConfigurationProcessor configurationProcessor;

    /** A {@link ProfileManager profile manager} */
    ProfileManager profileManager;

    /** AARRMain main constructor (no configuration is supplied)  */
    public AARRMain() throws AARRException {
        //this(null);
    }

    /** AARR main method
     * @param args command line arguments. Only an argument is passed by the moment: the name of a file containing the configuration */
    public static void main(String[] args) throws AARRException {
        /** A {@link Diagnostic diagnostic module } */
        Diagnostic diagnosticModule = new Diagnostic();

        /** First, we initiate the configuration module */
        confFile = expArg(args);

        Diagnostic.writeLog(Diagnostic.INFO_MODE,
            "Initiating configuration module");

        ConfigurationProcessor configurationProcessor = new ConfigurationProcessor(confFile);

        /** After initiating the configuration, we initiate a profile manager and get the
         * configured protocol adaptor */
        ProfileManager profileManager = new ProfileManager(configurationProcessor);

        Diagnostic.writeLog(Diagnostic.INFO_MODE,
            "Initiating the protocol adaptor module");

        protocolAdaptor = profileManager.initProtocolAdaptor();

        /** Then, we init the {@link RuleProcessor rule processor} */
        Diagnostic.writeLog(Diagnostic.INFO_MODE,
            "Initiating the rule processor module");

        ruleProcessor = profileManager.initRuleProcessor();
        ruleProcessor.setTimeout(configurationProcessor.getRulesetTimeout());

        // Now we are ready to plumb the protocol adaptor and the rule processor
        Diagnostic.writeLog(Diagnostic.INFO_MODE,
            "Plumbing the protocol adaptor and the rule processor");
        profileManager.plumb(protocolAdaptor, ruleProcessor); // que se "conozcan los objetos"
    }

    /**
     * read arguments, else print usage.
     */
    private static String expArg(String[] args) {
        if (args.length >= 1) {
            if (confFile == null) {
                confFile = args[0];
            } else {
                printUsage();
            }
        }

        if (confFile == null) {
            printUsage();
        }

        return confFile;
    }

    /** Prints usage if called from the command line. */
    private static void printUsage() {
        Diagnostic.writeLog(Diagnostic.WARNING_MODE,
            "Usage: <JavaVM> [ aarr | aarr.jar ] <confFile>");
    }
}
