/*
 * Class: ConfigurationProcessor
 *
 * $Id: ConfigurationProcessor.java,v 1.1.1.1 2005/05/22 17:00:49 macias Exp $
 */
package es.rediris.aarr;

import es.rediris.aarr.data.*;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import org.xml.sax.SAXException;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import java.util.Hashtable;
import java.util.Properties;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;


/**
 * This class is used for loading a given configuration specified from the command line.
 *
 * @author kan, macias
 *
 */
public class ConfigurationProcessor {
    /** Default name for the configuration file. */
    private static String DEFAULT_CONFIG_FILE = "aarr.conf";

    /** TODO: Default properties for AARR */
    protected Properties properties;

    /** An XML document */
    private Document docXml;

    /** The ruleset associated with the configuration */
    private Element ruleSet;

    /** A binding for this config. */
    private String binding;

    /** Environment section of the configuration */
    private Element environment;

    /** The protocol used in the configuration */
    private Element confProtocol;

    /** External datasource (to be defined the type at parseEnvironment) */
    private Hashtable dataSources = new Hashtable();

    /** Value of the instance as specified in the configuration */
    private String instance = null;

    /** Value of the working dir as specified in the configuration */
    private String workingDir = null;

    /** Value in milliseconds for the tiemout of the rule processor */
    private long ruleProcTimemout = Long.MAX_VALUE;

    /** Constructs the class and tries to parse the default config file */
    public ConfigurationProcessor() throws ConfigurationProcessorException {
        this("");
    }

    /** Constructs the class and tries to parse the given filename using the
     * {@link ConfigurationProcessor#parseXml parseXml()} method.
     * @param fileName the name of the file containing the configuration */
    public ConfigurationProcessor(String fileName)
        throws ConfigurationProcessorException {
        /// properties handling
        if (fileName.equals("")) {
            // TODO
        }

        try {
            File fileXml = new File(fileName);
            parseXml(fileXml);
        } catch (java.lang.NullPointerException e) {
            Diagnostic.writeLog(Diagnostic.WARNING_MODE,
                "ConfigurationProcessor: Error trying to open configuration file " +
                fileName);
            throw new ConfigurationProcessorException(
                "When trying to open configuration file " + fileName);
        }
    }

    /** Builds an XML Document from the supplied file and calls the
     *  {@link ConfigurationProcessor#parseProtocol parseProtocol()},
     *  {@link ConfigurationProcessor#parseEnvironment parseEnvironment()}
     *  and {@link ConfigurationProcessor#parseRuleset parseRuleset()} methods
     *  for parsing the different sections of the given XML file.
     *
     *  @param file a <code>java.io.File</code> object
     */
    private void parseXml(File file) throws ConfigurationProcessorException {
        docXml = null;

        try {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder db = dbf.newDocumentBuilder();
            docXml = db.parse(file);

            parseProtocol();
            parseEnvironment();
            ruleSet = parseRuleset();
        } catch (ParserConfigurationException e) {
            throw new ConfigurationProcessorException(
                "Error while parsing XML configuration file");
        } catch (SAXException e) {
            Diagnostic.writeLog(Diagnostic.SEVERE_MODE,
                "ConfigurationProcessor: Invalid XML file.");
            throw new ConfigurationProcessorException(
                "Error while parsing XML configuration file");
        } catch (IOException e) {
            Diagnostic.writeLog(Diagnostic.SEVERE_MODE,
                "ConfigurationProcessor: IO error when trying to parse configuration file.");
            throw new ConfigurationProcessorException(
                "IO error when trying to parse configuration file");
        }
    }

    /** Parses the protocol part of the configuration, creating a new {@link Configuration configuration}
     * with the provided binding and protocol */
    private void parseProtocol() throws ConfigurationProcessorException {
        Element elem;

        try {
            elem = (Element) org.apache.xpath.XPathAPI.selectSingleNode(docXml,
                    "/config/aarrdef");
        } catch (TransformerException e) {
            Diagnostic.writeLog(Diagnostic.SEVERE_MODE,
                "ConfigurationProcessor: Error while parsing the protocol section from configuration file.");
            throw new ConfigurationProcessorException(
                "Error while extracting protocol from configuration file.");
        }

        if (elem == null) {
            Diagnostic.writeLog(Diagnostic.SEVERE_MODE,
                "ConfigurationProcessor: No aarrdef element in configuration file.");
            throw new ConfigurationProcessorException(
                "No aarrdef element in configuration file.");
        }

        binding = elem.getAttribute("binding");

        if (binding == null) {
            Diagnostic.writeLog(Diagnostic.SEVERE_MODE,
                "ConfigurationProcessor: No binding name specified.");
            throw new ConfigurationProcessorException(
                "A name for the binding was not specified.");
        } else {
            try {
                elem = (Element) org.apache.xpath.XPathAPI.selectSingleNode(docXml,
                        "/config/protocol");
                confProtocol = elem;
            } catch (TransformerException e) {
                Diagnostic.writeLog(Diagnostic.INFO_MODE,
                    "No protocol section was found in the configuration.");
            }
        }
    }

    /** Parses the environment section of the configuration, storing keys and values */
    private void parseEnvironment() throws ConfigurationProcessorException {
        /** The Element containing the entire environment tree branch */
        Element envElement;

        try {
            envElement = (Element) org.apache.xpath.XPathAPI.selectSingleNode(docXml,
                    "/config/environment");
        } catch (TransformerException e) {
            Diagnostic.writeLog(Diagnostic.SEVERE_MODE,
                "ConfigurationProcessor: Error while parsing the protocol section from configuration file.");
            throw new ConfigurationProcessorException(
                "Error while extracting environment information from the configuration file.");
        }

        if (envElement == null) {
            Diagnostic.writeLog(Diagnostic.SEVERE_MODE,
                "ConfigurationProcessor: No environment section in configuration file.");

            throw new ConfigurationProcessorException(
                "No environment section in configuration file.");
        } else {
            try {
                Element instanceElem = (Element) org.apache.xpath.XPathAPI.selectSingleNode(docXml,
                        "/config/environment/instance");
                instance = instanceElem.getAttribute("type");
            } catch (TransformerException e) {
                Diagnostic.writeLog(Diagnostic.WARNING_MODE,
                    "ConfigurationProcessor: No 'instance' tag found at the environment section.");
            }

            try {
                Element workingdirElem = (Element) org.apache.xpath.XPathAPI.selectSingleNode(docXml,
                        "/config/environment/workingdir");
                workingDir = workingdirElem.getAttribute("path");
            } catch (TransformerException e) {
                Diagnostic.writeLog(Diagnostic.WARNING_MODE,
                    "ConfigurationProcessor: No 'workingdir' tag found at the environment section.");
            }

            try {
                Element timeoutElem = (Element) org.apache.xpath.XPathAPI.selectSingleNode(docXml,
                        "/config/environment/timeout");

                ruleProcTimemout = (1000 * (new Long(timeoutElem.getAttribute(
                            "seconds")).longValue())) +
                    new Long(timeoutElem.getAttribute("miliseconds")).longValue();
                Diagnostic.writeLog(Diagnostic.INFO_MODE,
                    "Timeout for the rule processor is " + ruleProcTimemout +
                    " milliseconds.");
            } catch (TransformerException e) {
                Diagnostic.writeLog(Diagnostic.WARNING_MODE,
                    "ConfigurationProcessor: No timeout tag found at the environment section. Using default timeout (" +
                    Long.MAX_VALUE + ").");
            }

            NodeList dataSourcesList = envElement.getElementsByTagName(
                    "datasource");

            if (dataSourcesList == null) {
                Diagnostic.writeLog(Diagnostic.WARNING_MODE,
                    "ConfigurationProcessor: No 'datasources' found at the environment section or datasource wrongly specified.");
            } else {
                try {
                    if (dataSourcesList.getLength() > 0) {
                        for (int i = 0; i < dataSourcesList.getLength(); i++) {
                            Element dataSourceElement = (Element) dataSourcesList.item(i);
                            Diagnostic.writeLog(Diagnostic.INFO_MODE,
                                "Processing data source '" +
                                dataSourceElement.getAttribute("name") + "'.");

                            if (dataSourceElement.getAttribute("type").equals("file")) {
                                String pathToFile = (String) dataSourceElement.getAttribute(
                                        "path");

                                DataSource ds = new PlaintextDataFile(dataSourceElement.getAttribute(
                                            "name"),
                                        pathToFile.startsWith("/") ? pathToFile
                                                                   : (workingDir +
                                        "/" + pathToFile));

                                ds.init();

                                dataSources.put((String) dataSourceElement.getAttribute(
                                        "name"), ds);
                            } else {
                                Diagnostic.writeLog(Diagnostic.WARNING_MODE,
                                    "ConfigurationProcessor: Ignoring unknown data source " +
                                    dataSourceElement.getAttribute("type") +
                                    ".");
                            }
                        }
                    }
                } catch (DataSourceException e) {
                    Diagnostic.writeLog(Diagnostic.SEVERE_MODE,
                        "ConfigurationProcessor: Error while processing data source.");

                    throw new ConfigurationProcessorException(
                        "Error while setting up datasources." + e.getMessage());
                }
            }
        }
    }

    /** Parses the ruleset, loading it from a external file */
    private Element parseRuleset() throws ConfigurationProcessorException {
        Element internalRuleset = null;

        try {
            internalRuleset = (Element) org.apache.xpath.XPathAPI.selectSingleNode(docXml,
                    "/config/ruleset");
        } catch (TransformerException e) {
            Diagnostic.writeLog(Diagnostic.SEVERE_MODE,
                "ConfigurationProcessor: Error while extracting ruleset from configuration file.");
            throw new ConfigurationProcessorException(
                "Error while extracting ruleset from configuration file.");
        }

        if (internalRuleset == null) {
            Diagnostic.writeLog(Diagnostic.SEVERE_MODE,
                "ConfigurationProcessor: Null (non existing or invalid) ruleset.");
            throw new ConfigurationProcessorException(
                "Null (non existing or invalid) ruleset.");
        } else {
            if (internalRuleset.hasAttribute("path")) {
                Element externalRuleset = null;

                try {
                    File fileXml = new File(internalRuleset.getAttribute("path"));

                    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
                    DocumentBuilder db = dbf.newDocumentBuilder();
                    Document docXml = db.parse(fileXml);

                    Diagnostic.writeLog(Diagnostic.INFO_MODE,
                        "Loading ruleset from external file " + fileXml);

                    externalRuleset = docXml.getDocumentElement();
                } catch (FileNotFoundException e) {
                    Diagnostic.writeLog(Diagnostic.SEVERE_MODE,
                        "ConfigurationProcessor: Configuration file " +
                        internalRuleset.getAttribute("path") + " not found.");
                    throw new ConfigurationProcessorException(
                        "Configuration file " +
                        internalRuleset.getAttribute("path") + " not found.");
                } catch (Exception e) {
                    Diagnostic.writeLog(Diagnostic.SEVERE_MODE,
                        "ConfigurationProcessor: Problem loading ruleset from external file: ");
                    throw new ConfigurationProcessorException(
                        "Problem loading ruleset from external file " +
                        internalRuleset.getAttribute("path") + e.getMessage());
                }

                return externalRuleset;
            } else {
                return internalRuleset;
            }
        }
    }

    /** Returns a String containing the name of the binding for this configuration */
    public String getBinding() {
        return binding;
    }

    /** Returns the Ruleset associated with this configuration
     * @return the ruleset for this configuration  */
    public Element getRuleset() {
        return ruleSet;
    }

    /** Returns the instance stored at the environment section in a configuration
     * @return the instance for the actual configuration  */
    public String getInstance() {
        return instance;
    }

    /** Returns the workingdir stored at the environment section in a configuration
     * @return the working dir from the configuration  */
    public String getWorkingDir() {
        return workingDir;
    }

    /** Returns the timeout for the rule processor stored at the environment section in a configuration
     * @return the timeout in milliseconds  */
    public long getRulesetTimeout() {
        return ruleProcTimemout;
    }

    /** Returns the workingdir stored at the environment section in a configuration
     * @return the working dir from the configuration  */
    public String getDataValue(String datasource, String key)
        throws ConfigurationProcessorException {
        String foundKey = null;

        try {
            DataSource ds = (DataSource) dataSources.get(datasource);

            if (ds == null) {
                Diagnostic.writeLog(Diagnostic.SEVERE_MODE,
                    "ConfigurationProcessor: Datasource '" + datasource +
                    "' does not exist.");
                throw new ConfigurationProcessorException("Datasource '" +
                    datasource + "' does not exist.");
            } else {
                foundKey = ds.getValue(key);
            }
        } catch (DataSourceException e) {
            Diagnostic.writeLog(Diagnostic.SEVERE_MODE,
                "ConfigurationProcessor: Exception when trying to get value for key '" +
                key + "' from datasource '" + datasource + "'.");
            throw new ConfigurationProcessorException(
                "Exception when trying to get value for key '" + key +
                "' from datasource '" + datasource + "'.");
        }

        return foundKey;
    }

    /** Returns the protocol associated with the configuration
     * @return the protocol */
    public Element getConfigProtocol() {
        return confProtocol;
    }

    /** Sets the protocol in the configuration to the provided one
     * @param conf the protocol
     */
    public void setConfigProtocol(Element conf) {
        confProtocol = conf;
    }

    /** Gets a library configuration property
     *
     * @param  key      A property name
     * @return          The property's value, or null if the property isn't set
     *
     */
    public String getProperty(String key) {
        return properties.getProperty(key);
    }
}
