/*
 * Class: ProfileManager
 *
 * $Id: ProfileManager.java,v 1.3 2005/05/26 15:48:46 macias Exp $
 */
package es.rediris.aarr;

import es.rediris.aarr.protocol.ProtocolAdaptor;
import es.rediris.aarr.protocol.ProtocolAdaptorConfigException;
import es.rediris.aarr.protocol.saml.SAMLPostAdaptor;
import es.rediris.aarr.rules.*;

import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 * This class defines a profile manager. It is used to
 * instantiate a {@link ProtocolAdaptor ProtocolAdaptor}
 * and a {@link RuleProcessor RuleProcessor}, then, the queues
 * in both are inited.
 *
 * @author kan, macias
 */
public class ProfileManager {
    /** The SAML protocol */
    public final static String SAML_PROTOCOL = "saml";

    /** A {@link ProtocolAdaptor ProtocolAdaptor} */
    ProtocolAdaptor protAdapt;

    /** A {@link RuleProcessor RuleProcessor} */
    RuleProcessor ruleProc;
    ConfigurationProcessor confProc;

    /** Unique constructor for this class. It asigns the given
     * ProtocolAdaptor and RuleProcessor to the internal */
    public ProfileManager(ConfigurationProcessor config)
        throws ProfileManagerException {
        protAdapt = getProtocolAdaptor(config.getBinding());
        ruleProc = new RuleProcessor(config.getRuleset());
        confProc = config;
    }

    /** Init a protocol adaptor and return an instance of it
     @return A {@link ProtocolAdaptor ProtocolAdaptor} object instance */
    public ProtocolAdaptor initProtocolAdaptor() {
        Element protocolConfiguration = confProc.getConfigProtocol();

        NodeList confNodes = protocolConfiguration.getChildNodes();

        try {
            for (int i = 0; i < confNodes.getLength(); i++) {
                Node configElement = confNodes.item(i);

                if (configElement.getNodeType() == Node.ELEMENT_NODE) {
                    NamedNodeMap nodeMap = configElement.getAttributes();
                    String[] attNames = new String[nodeMap.getLength()];
                    String[] attValues = new String[nodeMap.getLength()];

                    for (int j = 0; j < nodeMap.getLength(); j++) {
                        attNames[j] = nodeMap.item(j).getNodeName();
                        attValues[j] = nodeMap.item(j).getNodeValue();
                    }

                    protAdapt.setParam(configElement.getNodeName(), attNames,
                        attValues);
                }
            }
        } catch (ProtocolAdaptorConfigException e) {
            Diagnostic.writeLog(Diagnostic.SEVERE_MODE,
                "Problem while configuring protocol adaptor:" + e);
        }

        protAdapt.init(this);

        return protAdapt;
    }

    /** Init the rule processor */
    public RuleProcessor initRuleProcessor() {
        ruleProc.init(this);

        return ruleProc;
    }

    /** Returns the {@link ProtocolAdaptor ProtocolAdaptor} for this profile
     * @return A {@link ProtocolAdaptor ProtocolAdaptor} object  */
    public ProtocolAdaptor getProtocolAdaptor() {
        return protAdapt;
    }

    /** Init queues for the {@link ProtocolAdaptor ProtocolAdaptor} and
     * the {@link RuleProcessor RuleProcessor} */
    public void plumb(ProtocolAdaptor protocolAdaptor,
        RuleProcessor ruleProcessor) {
        try {
            protocolAdaptor.setObserver(ruleProcessor);
        } catch (ProtocolAdaptorConfigException e) {
            Diagnostic.writeLog(Diagnostic.SEVERE_MODE,
                "Unable to set the rule processor as an observer of the protocol adaptor in use.");
        }
    }

    /** Returns the right protocol adaptor for the given name. In other case, throws a
     * new {@link ProfileManagerException ProfileManagerException} exception
     * @param name is the name of the protocol
     * @throws ProfileManagerException an exception thrown when the specified protocol is unknown */
    private ProtocolAdaptor getProtocolAdaptor(String name)
        throws ProfileManagerException {
        if (name.equals(SAML_PROTOCOL)) {
            return new SAMLPostAdaptor();
        } else {
            Diagnostic.writeLog(Diagnostic.SEVERE_MODE, "ProfileManager: Unknown protocol '"+name+"'");
            throw new ProfileManagerException("Undefined protocol '"+name+"'");
        }
    }
}
