/*
 * Interface: ProtocolAdaptor
 *
 * $Id: ProtocolAdaptor.java,v 1.1.1.1 2005/05/22 17:00:49 macias Exp $
 */
package es.rediris.aarr.protocol;

import es.rediris.aarr.ProfileManager;
import es.rediris.aarr.data.ObservableVector;

import org.w3c.dom.Element;

import java.util.Observer;


/**
 *  This interface defines the methods that any implementing protocol adaptor must contain:
 * <ul>
 *   <li>{@link ProtocolAdaptor#setParam setParam()} to set up configuration params for this adaptor
 *   <li>{@link ProtocolAdaptor#init init()} to init the adaptor with the profile in use
 *   <li>{@link ProtocolAdaptor#setObserver setObserver()} to configure out an observer (the rule processor) to whom the adaptor will notify
 *   <li>{@link ProtocolAdaptor#getNextInput getNextInput()} for returning the request (input) that must be processed
 *   <li>{@link ProtocolAdaptor#getValue getValue()} Returns an array of values for the provided param name
 *   <li>{@link ProtocolAdaptor#discard discard()} Discards the active request
 *   <li>{@link ProtocolAdaptor#sendElement sendElement()} for receiving the components and values that are necessary to create a response
 * </ul>
 * @author macias
 */
public interface ProtocolAdaptor {
    /** Inits the adaptor with the information provided by the {@link ProfileManager ProfileManager} in use */
    public void init(ProfileManager pm);

    /** This method is used to set attributes and their respective values
     * for a given param <code>name</code>
     *
     * @param name The name for the param
     * @param ids An array of strings with identificators for the attributes
     * @param values An array of strings with the values for the attributes */
    public void setParam(String name, String[] ids, String[] values)
        throws ProtocolAdaptorConfigException;

    /** Sets an observer (the rule processor) for the protocol adaptor in use
     * @param obs An <code>java.util.Observer</code> object */
    public void setObserver(Observer obs) throws ProtocolAdaptorConfigException;

    /** Provides the label of the first available input element, and then
     * removes it; the element becomes the active one and overrides the previous.
     *
     * @return The name of the first available input element */
    public String getNextInput() throws ProtocolAdaptorReadException;

    /** Discards the active element and writes a diagnostic */
    public void discard();

    /** Gets the values for a given param of the active element
     * @param name the name of the param
     * @return An strings array with all the values for the param */
    public String[] getValue(String name);

    /** Sends an element with the corresponding components and values
     * @param name tha name of the element to be sent
     * @param components An array of strings containing the components of the element to be sent
     * @param values The values associated to the components */
    public void sendElement(String name, String[] components, String[] values)
        throws ProtocolAdaptorSendException;
}
