/*
 * Class: ConditionField
 *
 * $Id: ConditionField.java,v 1.1.1.1 2005/05/22 17:00:49 macias Exp $
 */
package es.rediris.aarr.rules;

import es.rediris.aarr.Diagnostic;

import org.w3c.dom.Element;

import java.util.Arrays;
import java.util.StringTokenizer;


/**
 *  This class extends a {@link Condition Condition} and is used to create and modify
 * a condition field. The class stores the name of the condition, the field attribute,
 * a value, the base and a key.
 *
 * @author kan, macias
 */
public class ConditionField extends Condition {
    /** The name for dis condition */
    String name;

    /** The name of a field */
    String field;

    /** The value associated with this ConditionField */
    String value;

    /** The base for this ConditionField */
    String base;

    /** The key of this ConditionField */
    String key;

    /** The constructor for this class receives a <code>org.w3c.dom.Element</code> that should
    contain five different attributes ("name", "field", "value", "base", and "key"). The purpose
    is to store these values from this attributes.

    @param cond A <code>org.w3c.dom.Element</code> specifying a condition */
    public ConditionField(Element cond) {
        String name = cond.getAttribute("name");

        if (name == null) {
            name = "";
        }

        setName(name);

        String field = cond.getAttribute("field");
        setField(field);

        String value = cond.getAttribute("value");

        if (value == null) {
            value = "";
        }

        setValue(value);

        String base = cond.getAttribute("base");

        if (base == null) {
            base = "";
        }

        setBase(base);

        String key = cond.getAttribute("key");

        if (key == null) {
            key = "";
        }

        setKey(key);
    }

    /** Gets the name for this condition

    @return A <code>java.lang.String</code> with the name of the condition
    */
    public String getName() {
        return name;
    }

    /** Sets a new name for this condition

    @param name A string with the new name */
    public void setName(String name) {
        this.name = name;
    }

    /** Gets the value for the field attribute
    @return A String with the value of the field variable */
    public String getField() {
        return field;
    }

    /** Sets the value of field
    @param field a String with the value for the field variable */
    public void setField(String field) {
        this.field = field;
    }

    /** Gets the value variable of this condition

    @return A String with the value for this condition */
    public String getValue() {
        return value;
    }

    /** Sets the value for this condition

    @param value A String with the new value */
    public void setValue(String value) {
        this.value = value;
    }

    /** Gets the base for this condition

    @return A <code>java.lang.String</code> with the base of this condition */
    public String getBase() {
        return base;
    }

    /** Sets a new base for the condition

    @param base A <code>java.lang.String</code> with the new base */
    public void setBase(String base) {
        this.base = base;
    }

    /** Sets a new key for the condition

    @return A <code>java.lang.String</code> with the value of the key */
    public String getKey() {
        return key;
    }

    /** Sets a new key for the condition

    @param key A <code>java.lang.String</code> with the new key */
    public void setKey(String key) {
        this.key = key;
    }

    /** Always true, since a "field" condition always need toi be queried

    @return A boolean value (always true) */
    public boolean needsAQuery() {
        return true;
    }

    /** Checks if the condition is true by performing a binary search of all the possible values in
    the array of attributes obtained from the {@link RuleProcessor#getParam(String) RuleProcessor.getParam() }
    for the field value in this Condition. On success (any of the values match) true is returned, otherwise
    false.

    @return True if the condition is true, false otherwise.  */
    public boolean checkCondition() {
        Object data = RuleProcessor.getParam(getField());

        if (data != null) {
            boolean isOk = true;

            if (data instanceof String[]) {
                String[] attrs = (String[]) data;

                if (getKey().equals("") && getBase().equals("")) {
                    StringTokenizer st = new StringTokenizer(getValue(), ",");

                    while (st.hasMoreTokens()) {
                        int search = Arrays.binarySearch(attrs,
                                (String) st.nextToken());
                        isOk &= (search >= 0);
                    }
                }
            } else {
                isOk = data.equals(getValue());
            }

            if (isOk) {
                String message = "Condition '" + this.name + "' is TRUE";
                Diagnostic.writeLog(Diagnostic.INFO_MODE, message);
            } else {
                String message = "Condition '" + this.name + "' is FALSE";
                Diagnostic.writeLog(Diagnostic.INFO_MODE, message);
            }

            return isOk;
        }

        return false;
    }
}
